/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014, 2015 (c)
 * 
 *    file: grid.c
 *    This file is part of the GBall project.
 *
 *    GBall is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    GBall is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with GBall.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include "defs.h"

extern char *grid_buf;

void create_outer_border(int j)
{
  int i;
  memset(grid_buf, 0, (SCREEN_H+1)*(SCREEN_W+1));
  for(i = 1; i <= SCREEN_W; i++) grid_buf[SCREEN_W+i]   = 1;
  for(i = 1; i <= SCREEN_W; i++) grid_buf[i+j]          = 1;
  for(i = 1; i <= SCREEN_H; i++) grid_buf[i*SCREEN_W-1] = 1;
  for(i = 1; i <= SCREEN_H; i++) grid_buf[i*SCREEN_W]   = 1;
}

void make_grid(int level)
{
  int i, j = SCREEN_H*SCREEN_W;
  int k = 0, l = SCREEN_W-1;
  int m, brick;
  if(((SCREEN_W-2)%8))
  {
    j = ((SCREEN_W-2)%8);
    l = SCREEN_W-1;
    if(j > 1) k = j/2;
    l -= (j-k);
    j = SCREEN_H*SCREEN_W;
  }
  k++;
  total_bricks = 0;

  total_block_lines = 8;
  blocks_per_line = (l-1)/8;
  line_start = k;
  
  switch(level)
  {
    case 1:
      /*
       * Level #1 grid:
       * +-----------------+
       * |*****************|
       * |*****************|
       * |                 |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_1;
      for(j = 2; j < 10; j++)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1) brick = BRICK_2;
	  else                 brick = BRICK_1;
	}
      }
      break;
    case 2:
      /*
       * Level #2 grid:
       * +-----------------+
       * |*****************|
       * |   ***********   |
       * |      *****      |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_1;
      for(j = 2; j < 10; j++, k+=8, l-=8)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_6;
	  else                      brick = BRICK_1;
	}
      }
      break;
    case 3:
      /*
       * Level #3 grid:
       * +-----------------+
       * |***   *****   ***|
       * |***   *****   ***|
       * |***   *****   ***|
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_1;
      //put the leftside column
      for(j = 2; j < 10; j++)
      for(i = k; i < k+24; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_4;
	  else if(brick == BRICK_4) brick = BRICK_1;
	}
      }
      //put the rightside column
      m = 0; brick = BRICK_1;
      for(j = 2; j < 10; j++)
      for(i = l-24; i < l; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_4;
	  else if(brick == BRICK_4) brick = BRICK_1;
	}
      }
      //put the middle column
      m = 0; brick = BRICK_1;
      k = SCREEN_W/2; k -= 24;
      for(j = 2; j < 10; j++)
      for(i = k; i < k+48; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_4;
	  else if(brick == BRICK_4) brick = BRICK_1;
	}
      }
      break;
    case 4:
      /*
       * Level #4 grid:
       * +-----------------+
       * |*****************|
       * |*****************|
       * |                 |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_3;
      for(j = 2; j < 10; j++, brick = (brick == BRICK_6) ? BRICK_3 : BRICK_6)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	}
      }
      break;
    case 5:
      /*
       * Level #5 grid:
       * +-----------------+
       * |***   *****   ***|
       * |***   *****   ***|
       * |***   *****   ***|
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_1;
      //put the leftside column
      for(j = 2; j < 10; j++)
      for(i = k; i < k+24; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_6;
	  else if(brick == BRICK_6) brick = BRICK_1;
	}
      }
      //put the rightside column
      m = 0; brick = BRICK_1;
      for(j = 2; j < 10; j++)
      for(i = l-24; i < l; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_6;
	  else if(brick == BRICK_6) brick = BRICK_1;
	}
      }
      //put the middle column
      m = 0; brick = BRICK_1;
      k = SCREEN_W/2; k -= 24;
      for(j = 2; j < 10; j++)
      for(i = k; i < k+48; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_1)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_6;
	  else if(brick == BRICK_6) brick = BRICK_1;
	}
      }
      break;
    case 6:
      /*
       * Level #6 grid:
       * +-----------------+
       * |*****************|
       * |   ***********   |
       * |      *****      |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_2;
      for(j = 2; j < 10; j++, k+=8, l-=8, brick = (brick == BRICK_6) ? BRICK_2 : BRICK_6)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	}
      }
      break;
    case 7:
      /*
       * Level #7 grid:
       * +-----------------+
       * |      *****      |
       * |      *****      |
       * |      *****      |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      //put the middle column
      m = 0; brick = BRICK_6;
      k = SCREEN_W/2; k -= 20;
      for(j = 2; j < 10; j++)
      for(i = k; i < k+48; i++)
      {
	if(j == 9) brick = 1;
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_6)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_3;
	  else if(brick == BRICK_3) brick = BRICK_6;
	}
      }
      break;
    case 8:
      /*
       * Level #8 grid:
       * +-----------------+
       * |*****************|
       * |*****************|
       * |                 |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_2;
      for(j = 2; j < 10; j++)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_2) brick = BRICK_6;
	  else                 brick = BRICK_2;
	}
      }
      break;
    case 9:
      /*
       * Level #9 grid:
       * +-----------------+
       * |   ***********   |
       * |      *****      |
       * |   ***********   |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      //put the bricks
      m = 0; brick = BRICK_6;
      for(j = 2; j < 6; j++, k+=8, l-=8)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_3)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_6;
	  else                      brick = BRICK_3;
	}
      }
      m = 0; //brick = BRICK_3;
      for(j = 6; j < 10; j++, k-=8, l+=8)
      for(i = k; i < l; i++)
      {
	grid_buf[SCREEN_W*j+i] = brick;
	m++;
	if(m > 7)
	{
	  m = 0; total_bricks++;
	  if(brick == BRICK_3)      brick = BRICK_2;
	  else if(brick == BRICK_2) brick = BRICK_6;
	  else                      brick = BRICK_3;
	}
      }
      break;
  }//end switch(level)
}//end make_grid()


/* This is used for debugging */

/*
void dump_grid()
{
  FILE *o = fopen("output", "w");
  j = 0;
  int pos = SCREEN_W;
  fputc('\n', stdout);
  for(i = 0; i < SCREEN_H; i++)
  {
    printf("%d/%d  [", i+1, SCREEN_H);
    //fprintf(o, "%d/%d  [", i+1, SCREEN_H);
    for(j = 0; j < SCREEN_W; j++)
    {
      if(grid_buf[pos] == 0) { fputc(' ', o); fputc('_', stdout); }
      else { fputc('*', o); fputc('*', stdout); }
      pos++;
    //fwrite(grid_buf+j, SCREEN_W, 1, o);
    //fwrite("\n", 1, 1, o);
    //j++;
    //j += SCREEN_W;
    }
    fputc('\n', o);
    fputc('\n', stdout);
  }
  while(getKey() != ENTER_KEY) ;
  fclose(o);
}
*/
