/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014, 2015 (c)
 * 
 *    file: main.c
 *    This file is part of the GBall project.
 *
 *    GBall is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    GBall is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with GBall.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include <getopt.h>
#include "defs.h"

void sighandler(int signo) 
{
  
}

void draw_ball();
void game_over();
void make_food();
void exit_game();
void load_level(int level);
extern void make_grid(int level);
extern int show_menu(char *title);

char *grid_buf;
int speed = 50000;
int MAX_SCORE = 10;
char *gball_ver = "2.0";


int main(int argc, char **argv)
{
  ///////////////////////////////////////
  //parse command line arguments
  ///////////////////////////////////////
  int START_LEVEL = 0;
  int c;
  static struct option long_options[] =
  {
    {"version", no_argument,       0,         'v'},
    {"help",    no_argument,       0,         'h'},
    {"level", required_argument,   0,         'l'},
    {0, 0, 0, 0}
  };
  
  while(1)
  {
    int option_index=0;
    c = getopt_long(argc, argv, "vhl:", long_options, &option_index);
    if(c==-1) break;	/* end of options */
    
    switch(c)
    {
      case 0:
	break;
      case 'v':		/* show version & exit */
	printf("%s\n", gball_ver);
	exit(0); break;
      case 'h':		/* show help & exit */
	printf("The Console GBall Game, version %s\n"
	       "Usage: %s [options]\n\n"
	       "Options:\n"
	       "-h, --help           Print help [this page]\n"
	       "-v, --version        Display program version\n"
	       , gball_ver, argv[0]);
	exit(0); break;
      case 'l':
	START_LEVEL = atoi(optarg);
	if(START_LEVEL < 1 || START_LEVEL > MAX_LEVEL)
	{
	  printf("Level must be between 1 and %d\n", MAX_LEVEL);
	  exit(1); break;
	}
      case '?':
	break;
      default:
	abort();
    }
  }
  ///////////////////////////////////////
  ///////////////////////////////////////
  if(!initTerminal())
  {
    printf("Error initializing the screen\n");
    exit(1);
  }
  int flags;
  /* make stdin non-blocking */
  flags = fcntl(0, F_GETFL);
  flags |= O_NONBLOCK;
  fcntl(0, F_SETFL, flags);

  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  clearScreen();
  getScreenSize();
  drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
  /* turn the cursor off */
  printf("\e[?25l");
  
  /* initialize grid */
  grid_buf = (char *)malloc((SCREEN_H+1)*(SCREEN_W+1));
  if(!grid_buf)
  {
    printf("Error: Insufficient memory\n");
    exit_game();
  }
  int i;
  char grid[SCREEN_H+1][SCREEN_W+1];
  srand(time(NULL));
  if(START_LEVEL)
  {
    load_level(START_LEVEL);
    memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
  }
  else
  {
    i = show_menu("GBall");
    if(i) memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
  }
  locate(1, 3); 	     printf("SCORE: %04d", score);
  locate(1, (SCREEN_W/2)-6); printf("BRICKS: %04d", total_bricks);
  locate(1, SCREEN_W-10);    printf("LEVEL: %02d", LEVEL);

  /* Loop until we get a hit on ENTER */
  long int wait = speed*3;
  while(1)
  {
    /* ball hits racket */
    if(ball.x == racket.x-1)
    {
      if((ball.direction == DOWN_RIGHT) &&
	(ball.y >= racket.y-1 && ball.y <= racket.length+racket.y-1))
      { ball.direction = UP_RIGHT; draw_ball(); }
      else if((ball.direction == DOWN_LEFT) &&
	(ball.y >= racket.y+1 && ball.y <= racket.length+racket.y+1))
      { ball.direction = UP_LEFT; draw_ball(); }
    }
    
    //////////////////////////////////////////
    // Check if we finished all bricks
    //////////////////////////////////////////
    if(total_bricks <= 0)
    {
      LEVEL++;
      if(LEVEL > MAX_LEVEL) LEVEL = 1;
      MAX_SCORE = DEFAULT_MAX_SCORE;
      MAX_SCORE += 5;
      speed -= 5000;
      level_score = 0;
      racket.length = 8;
      racket.x = SCREEN_H-1;
      racket.y = (SCREEN_W/2);
      racket.y -= 4;
      racket.dy = SCREEN_W/20;
      make_grid(LEVEL);		/* make grid for new level */
      ball.x = ball.old_x = racket.x-1;
      ball.y = ball.old_y = racket.y+3;
      ball.direction = rand() % 3;
      ball.direction++;
      drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
      draw_ball();
      usleep(speed);
    }
    if(--wait) goto skip;
    wait = speed*3;
    
    ball.old_x = ball.x;
    ball.old_y = ball.y;
    if(ball.direction == UP_LEFT)
    {
      ball.x--; ball.y--; 
      if(ball.x == 1)
      {
	ball.direction = DOWN_RIGHT;
	ball.x += 2;
      }
    }
    else if(ball.direction == UP_RIGHT)
    {
      ball.x--; ball.y++; 
      if(ball.x == 1)
      {
	ball.direction = DOWN_RIGHT;
	ball.x += 2;
      }
    }
    else if(ball.direction == DOWN_LEFT)
    {
      ball.x++; ball.y--;
      if(ball.y == 1)
      {
	ball.direction = DOWN_RIGHT;
	ball.y += 2;
      }
    }
    else if(ball.direction == DOWN_RIGHT)
    {
      ball.x++; ball.y++;
      if(ball.y == SCREEN_W)
      {
	ball.direction = DOWN_LEFT;
	ball.y -= 2;
      }
    }
    draw_ball();
    
    int ch;
skip:
    ch = getKey();
    if(ch == LEFT_KEY || ch == 'a')
    {
      if(racket.y > 2)
      {
	/* remove old racket */
	setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
	locate(racket.x, racket.y);
	printf("%*s", racket.length, " ");
	/* update racket location */
	racket.y -= racket.dy;
	if(racket.y < 2) racket.y = 2;
      }
      else goto sleep;
      draw_ball();
    }
    else if(ch == RIGHT_KEY || ch == 'd')
    {
      if(racket.y+racket.length < SCREEN_W)
      {
	/* remove old racket */
	setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
	locate(racket.x, racket.y);
	printf("%*s", racket.length, " ");
	/* update racket location */
	racket.y += racket.dy;
	if(racket.y+racket.length >= SCREEN_W)
	  racket.y = SCREEN_W-racket.length;
      }
      else goto sleep;
      draw_ball();
    }
    else if(ch == ENTER_KEY || ch == SPACE_KEY || ch == 'p')
    {
      i = show_menu("PAUSED");
      if(i) memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
    }
    else if(ch == '0')
    {
      speed -= SPEED_STEP;
      if(speed < MAX_SPEED) speed = MAX_SPEED;
    }
    else if(ch == '9')
    {
      speed += SPEED_STEP;
      if(speed > MIN_SPEED) speed = MIN_SPEED;
    }
    else if(ch == ESC_KEY) break;
sleep:
    continue;
    //usleep(speed);
  } /* end while */

  exit_game();
  return 0;
}


/*
 * Draw the ball to screen.
 */
void draw_ball()
{
  int last_color = BG_COLOR[COLOR_WINDOW];
  
  setScreenColors(BLACK, BGWHITE);
  locate(racket.x, racket.y);
  printf("%*s", racket.length, " ");
  setScreenColors(WHITE, last_color);
  locate(ball.old_x, ball.old_y);
  putchar(' ');
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  locate(ball.x, ball.y);
  putchar('*');
  fflush(stdout);
  
  /* check for brick hits */
  int collision = 1;	/* assume collision for starters */
  int is_brick = 1;	/* did we hit a brick or a wall? */
  int brick_x = 0, brick_y = 0;
  if(ball.direction == UP_LEFT || ball.direction == UP_RIGHT)
  {
    int index = (SCREEN_W*(ball.x-1))+ball.y-1;
    if(grid_buf[index] >= 1)
    {
      if(ball.direction == UP_LEFT) ball.direction = DOWN_LEFT;
      else if(ball.direction == UP_RIGHT) ball.direction = DOWN_RIGHT;
      brick_x = ball.x-1; brick_y = ball.y-1;
      if(grid_buf[index] == 1)
      {
	is_brick = 0;
      }
    }
    else if(ball.direction == UP_LEFT)
    {
      index = (SCREEN_W*ball.x)+ball.y-2;
      if(grid_buf[index] >= 1)
      {
	ball.direction = UP_RIGHT;
	brick_x = ball.x; brick_y = ball.y-2;
	if(grid_buf[index] == 1) is_brick = 0;
      }
      else collision = 0;
    }
    else if(ball.direction == UP_RIGHT)
    {
      index = (SCREEN_W*ball.x)+ball.y;
      if(grid_buf[index] >= 1)
      {
	ball.direction = UP_LEFT;
	brick_x = ball.x; brick_y = ball.y;
	if(grid_buf[index] == 1) is_brick = 0;
      }
      else collision = 0;
    }
  }
  else		/* means going down */
  {
    int index = (SCREEN_W*(ball.x+1))+ball.y-1;
    if(grid_buf[index] >= 1)
    {
      if(ball.direction == DOWN_LEFT) ball.direction = UP_LEFT;
      else if(ball.direction == DOWN_RIGHT) ball.direction = UP_RIGHT;
      brick_x = ball.x+1; brick_y = ball.y-1;
      if(grid_buf[index] == 1 && ball.x == SCREEN_H-1)
      {
	game_over();
	return;
      }
    }
    else if(ball.direction == DOWN_LEFT)
    {
      index = (SCREEN_W*ball.x)+ball.y-2;
      if(grid_buf[index] >= 1)
      {
	ball.direction = DOWN_RIGHT;
	brick_x = ball.x; brick_y = ball.y-2;
	if(grid_buf[index] == 1) is_brick = 0;
      }
      else collision = 0;
    }
    else if(ball.direction == DOWN_RIGHT)
    {
      index = (SCREEN_W*ball.x)+ball.y;
      if(grid_buf[index] >= 1)
      {
	ball.direction = DOWN_LEFT;
	brick_x = ball.x; brick_y = ball.y;
	if(grid_buf[index] == 1) is_brick = 0;
      }
      else collision = 0;
    }
    else collision = 0;
  }

  if(collision && is_brick)
  {
    int a = brick_y-line_start;
    a /= 8;
    int b = a*8;
    int c = b+line_start;
    int d;
    int index = (SCREEN_W*brick_x);
    score += grid_buf[index+brick_y]-40;
    level_score += grid_buf[index+brick_y]-40;
    if(grid_buf[index+brick_y] == BRICK_6)
    {
      for(d = 0; d < 8; d++)
	grid_buf[index+c+d] = BRICK_4;
      setScreenColors(WHITE, BRICK_4);
      last_color = BRICK_4;
    }
    else
    {
      for(d = 0; d < 8; d++)
	grid_buf[index+c+d] = 0;
      total_bricks--;
      setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
      last_color = BG_COLOR[COLOR_WINDOW];
    }
    c++;
    for(d = 0; d < 8; d++)
    {
      locate(brick_x, c+d);
      putchar(' ');
    }
  }

  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  locate(1, 3); 	     printf("SCORE: %04d", score);
  locate(1, (SCREEN_W/2)-6); printf("BRICKS: %04d", total_bricks);
  locate(1, SCREEN_W-10);    printf("LEVEL: %02d", LEVEL);
  setScreenColors(BLACK, BGWHITE);
  int i = SCREEN_W*2, j, k;
  for(j = 2; j < SCREEN_H; j++)
  {
    i++;
    for(k = 1; k < SCREEN_W-1; k++)
    {
      if(grid_buf[i] == BRICK_1)      setScreenColors(BLACK, BRICK_1);
      else if(grid_buf[i] == BRICK_2) setScreenColors(BLACK, BRICK_2);
      else if(grid_buf[i] == BRICK_3) setScreenColors(BLACK, BRICK_3);
      else if(grid_buf[i] == BRICK_4) setScreenColors(BLACK, BRICK_4);
      else if(grid_buf[i] == BRICK_5) setScreenColors(BLACK, BRICK_5);
      else if(grid_buf[i] == BRICK_6) setScreenColors(BLACK, BRICK_6);
      else if(grid_buf[i] == BRICK_7) setScreenColors(BLACK, BRICK_7);
      else if(grid_buf[i] == 1)       setScreenColors(BLACK, BGWHITE);
      else { i++; continue; }
      locate(j, k+1);
      putchar(32);
      i++;
    }
    i++;
  }
}

/*
 * Exit the game safely, clearing the screen and restoring the terminal.
 */
void exit_game()
{
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  clearScreen();
  restoreTerminal();
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  fprintf(stdout, "\x1b[24m");
  fflush(stdout);
  /* 
   * erase display buffer, then turn the cursor on.
   * FIXME: Currently this doesn't erase the buffer, why?.
   */
  printf("\e[3J");
  printf("\e[?25h");
  fflush(stdout);
  exit(0);
}

/*
 * Show the "GAME OVER" box
 */
void game_over()
{
  char *file_name;
  FILE *file;
  int y = (SCREEN_W/2)-14;
  int x = (SCREEN_H/2)-3;
  int w = y+28, h = x+5;
  int i;
  if(!score) goto show_game_over_box;
  
  if(!(pass = getpwuid(geteuid())))
  {
    goto show_game_over_box;
  }
  file_name = (char *) malloc(strlen(pass->pw_dir)+10);
  if(!file_name)
  {
    msgBox("Error: Insufficient memory", OK, ERROR);
    goto show_game_over_box;
  }
  strcpy(file_name, pass->pw_dir);
  strcat(file_name, "/");
  strcat(file_name, ".gball");
  if(!(file = fopen(file_name, "r+")))
  {
    //create a new highscores file
    if(!(file = fopen(file_name, "w+")))
    {
      goto show_game_over_box;
    }
    for(i = 0; i < 10; i++)
      fprintf(file, "%4d NONE             0000\n", i+1);
    fflush(file); rewind(file);
  }
  
  //read highscores
  char buf[40];
  char score_str[5];
  int index;
  for(i = 0; i < 10; i++)
  {
    if(!fgets(buf, 40, file)) break;
    buf[26] = '\0';
    score_str[0] = buf[22]; score_str[1] = buf[23]; score_str[2] = buf[24];
    score_str[3] = buf[25]; score_str[4] = '\0';
    if(score >= atoi(score_str))
    {
      //turn the cursor on
      printf("\e[?25h");
      index = i;
      inputBox("Enter your name:      ", "New Highscore!");
      if(!strlen(input)) strcpy(input, "NONE\0");
      if(strlen(input) > 16) input[16] = '\0';
      //turn the cursor off
      printf("\e[?25l");
      fseek(file, 0, SEEK_END);
      long file_pos = ftell(file);
      char *buf2 = (char *)malloc(file_pos);
      char *buf3 = (char *)malloc(40);
      if(!buf2 || !buf3)
      {
	msgBox("Error: Insufficient memory", OK, ERROR);
	goto show_game_over_box;
      }
      rewind(file);
      if(fread(buf2, 1, file_pos, file) != file_pos)
      {
	msgBox("Error reading highscores file", OK, ERROR);
	goto show_game_over_box;
      }
      rewind(file);
      i = 0;
      if(index)
      {
	buf3 = strtok(buf2, "\n");
	buf3[3] = '1';
	fprintf(file, "%s\n", buf3);
	for(i = 1; i < index; i++)
	{
	  buf3 = strtok(NULL, "\n");
	  buf3[3] = i+'1';
	  fprintf(file, "%s\n", buf3);
	}
      }
      
      fprintf(file, "%4d %*s %04d\n", index+1, -16, input, score);
      fflush(file);
      if(index) index++;
      else 
      {
	buf3 = strtok(buf2, "\n");
	buf3[3] = '2';
	fprintf(file, "%s\n", buf3);
	index = 2;
      }
      
      for(i = index; i < 10; i++)
      {
	buf3 = strtok(NULL, "\n");
	if(i == 9) { buf3[2] = '1'; buf3[3] = '0'; }
	else buf3[3] = i+'1';
	fprintf(file, "%s\n", buf3);
      }
      
      fflush(file); rewind(file);
      //draw box
      y = (SCREEN_W/2)-15;
      x = (SCREEN_H/2)-7;
      w = y+30, h = x+13;
      drawBox(x, y, h, w, "HIGHSCORES", YES);
      //read highscores
      for(i = 0; i < 10; i++)
      {
	if(!fgets(buf, 40, file)) break;
	buf[26] = '\0';
	locate(x+i+2, y+2);
	printf("%s", buf);
      }
      fflush(stdout);
      //loop for user pressing ENTER, SPACE, or ESC
      while(1)
      {
	char c = getKey();
	if(c == ENTER_KEY || c == SPACE_KEY || c == ESC_KEY) break;
      }
      fclose(file);
      break;
    }
  }
  show_menu("GBall");
  return;
show_game_over_box:
  show_menu("GAME OVER");
  return;
}


void load_level(int level)
{
  LEVEL = level;
  MAX_SCORE = DEFAULT_MAX_SCORE;
  speed = BASE_SPEED;
  score = level_score = 0;
  racket.length = 8;
  racket.x = SCREEN_H-1;
  racket.y = (SCREEN_W/2);
  racket.y -= 4;
  racket.dy = SCREEN_W/20;
  make_grid(LEVEL);	//make grid for new level
  ball.x = ball.old_x = racket.x-1;
  ball.y = ball.old_y = racket.y+3;
  ball.direction = rand() % 3;
  ball.direction++;
  drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
  return;
}
