/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014, 2015 (c)
 * 
 *    file: menu.c
 *    This file is part of the GBall project.
 *
 *    GBall is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    GBall is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with GBall.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include "defs.h"

#define MAIN_MENU	1
#define PAUSED_MENU	2

extern void exit_game();
extern void make_grid(int level);
void show_highscores();

extern int speed;
extern int MAX_SCORE;

void create_new_game()
{
  LEVEL = 1;
  MAX_SCORE = DEFAULT_MAX_SCORE;
  speed = BASE_SPEED;
  score = level_score = 0;
  racket.length = 8;
  racket.x = SCREEN_H-1;
  racket.y = (SCREEN_W/2);
  racket.y -= 4;
  racket.dy = SCREEN_W/20;
  make_grid(LEVEL);	//make grid for new level
  ball.x = ball.old_x = racket.x-1;
  ball.y = ball.old_y = racket.y+3;
  ball.direction = rand() % 3;
  ball.direction++;
  drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
}

void create_random_game()
{
  LEVEL = rand() % MAX_LEVEL; LEVEL++;
  MAX_SCORE = DEFAULT_MAX_SCORE;
  speed = BASE_SPEED;
  score = level_score = 0;
  racket.length = 8;
  racket.x = SCREEN_H-1;
  racket.y = (SCREEN_W/2);
  racket.y -= 4;
  racket.dy = SCREEN_W/20;
  make_grid(LEVEL);	//make grid for new level
  ball.x = ball.old_x = racket.x-1;
  ball.y = ball.old_y = racket.y+3;
  ball.direction = rand() % 3;
  ball.direction++;
  drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
}


int show_menu(char *title)
{
  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
  clearScreen();
  char menu[5][13];
  int total_menu_items = 5;
  int selected_item = 0;
  int x = (SCREEN_H/2)-4;
  int y = (SCREEN_W/2)-8;
  int w = y+16, h = x+8;
  char menu_type;
  int i;
  //the PAUSED menu
  if(strcmp(title, "PAUSED") == 0)
  {
    strcpy(menu[0], "Continue    \0");
    strcpy(menu[1], "New Game    \0");
    strcpy(menu[2], "Random Game \0");
    strcpy(menu[3], "\0"            );
    strcpy(menu[4], "Exit        \0");
    menu_type = PAUSED_MENU;
  }
  //the MAIN menu
  else
  {
    strcpy(menu[0], "New Game    \0");
    strcpy(menu[1], "Random Game \0");
    strcpy(menu[2], "Highscores  \0");
    strcpy(menu[3], "\0"            );
    strcpy(menu[4], "Exit        \0");
    menu_type = MAIN_MENU;
  }
  //draw the menu box
  drawBox(x, y, h, w, title, YES);
  setScreenColors(BLACK, BGWHITE);
  locate(x+2, y+2);
  printf("%s", menu[0]);
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  for(i = 1; i < total_menu_items; i++)
  {
    locate(x+i+2, y+2);
    printf("%s", menu[i]);
  }
  fflush(stdout);
  /*
   * loop for user input
   */
  while(1)
  {
    char c = getKey();
    if(c <= 0) continue;
    switch(c)
    {
      case RIGHT_KEY:
      case DOWN_KEY:
	setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
	locate(x+selected_item+2, y+2);
	printf("%s", menu[selected_item]);
	selected_item++;
	if(selected_item == 3) selected_item++;
	if(selected_item >= total_menu_items) selected_item = 0;
	setScreenColors(BLACK, BGWHITE);
	locate(x+selected_item+2, y+2);
	printf("%s", menu[selected_item]);
	fflush(stdout);
	break;
      case LEFT_KEY:
      case UP_KEY:
	setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
	locate(x+selected_item+2, y+2);
	printf("%s", menu[selected_item]);
	selected_item--;
	if(selected_item == 3) selected_item--;
	if(selected_item < 0) selected_item = total_menu_items-1;
	setScreenColors(BLACK, BGWHITE);
	locate(x+selected_item+2, y+2);
	printf("%s", menu[selected_item]);
	fflush(stdout);
	break;
      case ESC_KEY:
	if(menu_type == MAIN_MENU)
	  exit_game();
	else
	  return 0;
	break;
      case SPACE_KEY:
      case ENTER_KEY:
	if(menu_type == PAUSED_MENU)
	{
	  if(selected_item == 0)	//continue
	  {
	    drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
	    return 0;
	  }
	  else if(selected_item == 1)	//new game
	  {
	    create_new_game();
	    return 1;
	  }
	  else if(selected_item == 2)	//random game
	  {
	    create_random_game();
	    return 1;
	  }
	  else if(selected_item == 4)	//exit
	    exit_game();
	  else continue;
	}//end if (PAUSED_MENU)
	///////////////////////////////////
	///////////////////////////////////
	//Main Menu
	else
	{
	  if(selected_item == 0)	//new game
	  {
	    create_new_game();
	    return 1;
	  }
	  else if(selected_item == 1)	//random game
	  {
	    create_random_game();
	    return 1;
	  }
	  else if(selected_item == 2)	//highscores
	  {
	    show_highscores();
	    setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
	    //redraw the menu box
	    clearScreen();
	    drawBox(x, y, h, w, title, YES);
	    for(i = 0; i < total_menu_items; i++)
	    {
	      if(i == selected_item) setScreenColors(BLACK, BGWHITE);
	      else setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
	      locate(x+i+2, y+2);
	      printf("%s", menu[i]);
	    }
	    fflush(stdout);
	  }
	  else if(selected_item == 4)	//exit
	    exit_game();
	  else continue;
	}
	break;
    }//end switch
    setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  }
}


void show_highscores()
{
  char *file_name;
  FILE *file;
  int y = (SCREEN_W/2)-15;
  int x = (SCREEN_H/2)-7;
  int w = y+30, h = x+13;
  int i;
  if(!(pass = getpwuid(geteuid())))
  {
    msgBox("Error: couldn't open home directory", OK, ERROR);
    return;
  }
  file_name = (char *) malloc(strlen(pass->pw_dir)+10);
  if(!file_name)
  {
    msgBox("Error: Insufficient memory", OK, ERROR);
    return;
  }
  strcpy(file_name, pass->pw_dir);
  strcat(file_name, "/");
  strcat(file_name, ".gball");
  if(!(file = fopen(file_name, "r+")))
  {
    //create a new highscores file
    if(!(file = fopen(file_name, "w+")))
    {
      msgBox("Error accessing config file in home directory.", OK, ERROR);
      return;
    }
    for(i = 0; i < 10; i++)
      if(fprintf(file, "%4d NONE             0000\n", i+1) < 0)
	goto file_error;
    if(fflush(file) != 0) goto file_error;
    rewind(file);
  }
  
  drawBox(x, y, h, w, "HIGHSCORES", YES);
  //read highscores
  char buf[40];
  for(i = 0; i < 10; i++)
  {
    if(!fgets(buf, 40, file)) break;
    buf[26] = '\0';
    locate(x+i+2, y+2);
    printf("%s", buf);
  }
  fflush(stdout);
  //loop for user pressing ENTER, SPACE, or ESC
  while(1)
  {
    char c = getKey();
    if(c == ENTER_KEY || c == SPACE_KEY || c == ESC_KEY) break;
  }
  fclose(file);
  return;

file_error:
  msgBox("Error writing to highscores file", OK, ERROR);
  fclose(file);
  return;
}
